@@ -212,7 +212,7 @@ class AGServerHelperToken(object):
                 subfolder_list = json.loads(data)
 
                 for single in subfolder_list["services"]:
-                    services.append(subfolder + "//" + single['serviceName'] + '.' + single['type'])
+                    services.append(subfolder + "/" + single['serviceName'] + '.' + single['type'])
 
         if not services:
             if self.Tool != None:
